#!/usr/bin/perl

# Copyright (c) 2009 Jonathan Kamens <jik@kamens.brookline.ma.us>.
#
# May be distributed under the terms of the GNU General Public
# License, Version 3 or any newer version at your discretion.  See
# http://www.gnu.org/licenses/gpl.html for license terms.

# Reads Outlook Contacts and vonage-cid.pl CSVs to get phone numbers
# and names.  Reads vonage voicemail notification on stdin email
# message and adds names to numbers whenever possible.  Sends filtered
# result to stdout.
#
# Use like this in .procmailrc:
#
# :0 Hhbwf :
# * Subject: (Vonage Visual Voicemail from .* - New Voicemail Received|You have received a new voicemail message)
# |$HOME/scripts/vonage-vmail-filter.pl
#
# Set config variables below to correct paths (or unset) as needed.
#
# $Id: vonage-vmail-filter.pl,v 1.4 2009/12/30 21:01:37 jik Exp $

# Unset this if you're not using vonage-cid.pl; otherwise set it to
# the file where you've got vonage-cid.pl saving caller ID info.

$cid_db = "$ENV{HOME}/random/caller-id-database";

# Unset this if you're not using Outlook Contacts; otherwise, set it
# to the path of the CSV file you exported from your Outlook Contacts.

$outlook_csv = "$ENV{HOME}/random/outlook-contacts.csv";

use HTML::Entities;
use Text::CSV_XS;

# Read caller ID log

if ($cid_db) {
    open(DB, "<", $cid_db) or die;
    my $csv = Text::CSV_XS->new({binary => 1}) or die;
    $csv->column_names($csv->getline(\*DB)) or die;
    while (my $row = $csv->getline_hr(\*DB)) {
	next if (! $row->{number});
	next if (! $row->{cid});
	&add_number($row->{number}, $row->{cid}, "Caller ID");
    }
    close(DB) or die;
}

# Read Outlook Contacts

if ($outlook_csv) {
    open(CSV, "<", $outlook_csv) or die;
    my $csv = Text::CSV_XS->new({binary => 1}) or die;
    $csv->column_names($csv->getline(\*CSV)) or die;
    while (my $row = $csv->getline_hr(\*CSV)) {
	my $name = join(" ", $row->{"First Name"}, $row->{"Middle Name"},
			$row->{"Last Name"});
	$name =~ s/\s\s+/ /g;
	$name =~ s/^\s+//;
	$name =~ s/\s+$//;
	if (! $name and $row->{"Company"}) {
	    $name = $row->{"Company"};
	    $name =~ s/\s\s+/ /g;
	    $name =~ s/^\s+//;
	    $name =~ s/\s+$//;
	}
	foreach my $number ("Assistant's Phone", "Business Phone",
			    "Business Phone 2", "Car Phone",
			    "Company Main Phone", "Home Fax", "Home Phone",
			    "Home Phone 2", "Mobile Phone", "Other Fax",
			    "Other Phone", "Pager", "Primary Phone",
			    "Radio Phone", "TTY/TDD Phone") {
	    if ($row->{$number}) {
		&add_number($row->{$number}, $name, $number);
	    }
	}
    }
}

# Now mess with incoming notification message

{
    local($/) = "\n\n";
    $header = <STDIN>;
    $/ = undef;
    $body = <STDIN>;
}

&do_pattern({ string => \$header,
	      pattern => qr/^Subject: Vonage Visual Voicemail from (\d+)/m });
if (my $number = &do_pattern({ string => \$body,
			       pattern => qr/^From: (\d+)/m,
			       verbose => 1 })) {
    &do_pattern({ string => \$body,
		  pattern => qr/<td>\s*From\s*<\/td>\s*<td>\s*:\s*(\d+)/,
		  verbose => 1,
		  html => 1 });
    &do_pattern({ string => \$header,
		  pattern => qr/^Subject: You have received a new voicemail message$/m,
		  number => $number });
}

print $header, $body;
exit;

sub do_pattern {
    my($params) = @_;
    my $string = $params->{string};
    my $pattern = $params->{pattern};
    my $verbose = $params->{verbose};
    my $number = $params->{number};
    my $html = $params->{html};
    die if (! ($string && $pattern));
    if ($$string =~ /$pattern/) {
	$number = &fix_number($1) if (! $number);
	if ($names{$number}) {
	    my $tag = " ($names{$number}";
	    if ($verbose) {
		$tag .= " ($types{$number})";
	    }
	    $tag .= ")";
	    if ($html) {
		$tag = encode_entities($tag);
	    }
	    $$string =~ s/($pattern)/$1$tag/;
	    return $number;
	}
    }
    return undef;
}


sub add_number {
    my($number, $name, $type) = @_;
    $number = &fix_number($number);
    $names{$number} = $name;
    $types{$number} = $type;
}

sub fix_number {
    local($_) = @_;
    s/(\d)\s*x\s*\d+$/$1/;
    s/(\d)\s*ext\.?\s*\d+$/$1/;
    s/\D+//g;
    s/^1//;
    $_;
}

# $Log: vonage-vmail-filter.pl,v $
# Revision 1.4  2009/12/30 21:01:37  jik
# Use Outlook "Company" field if name is not set.
# Put change log at bottom of script.
#
